.segment "GLUE"
.macpack atari

; DEBUG = 1

BANK_SELECTOR = $BF80

zp_temp       = $30
zp_temp2      = $32

PORTA         = $2F0
consol        = $2F1
kbcode        = $2F2
pending_key   = $2F3
pending_count = $2F4
ball_cx       = $2F5
ball_cy       = $2F6
pot2          = $2F7
pot3          = $2F8
crosshair_x   = $2F9
crosshair_y   = $2FA
penh          = $2FB
penv          = $2FC
erase_y       = $2FD

VVBLKI	= $0202
VVBLKD	= $0224

PLAYER_1_MEMORY     = $0A80
DEBUG_SCREEN_MEMORY = $1000

HPOSP1	= $C001
TRIG1 	= $C011
COLPM0	= $C012
PRIOR	 = $C01B
CONSOL = $C01F
DMACTL	= $D400
DLISTL	= $D402
DLISTH	= $D403
CHBASE	= $D409
POT2	  = $E802
POT3  	= $E803
KBCODE	= $E809
POTGO	 = $E80B
SKSTAT	= $E80F

; *****************************************************************************
; * Display lists moved from $BF75.. as the bank switching area is $BF80..    *
; *****************************************************************************
dlist1:
  .byte $70,$70
  .byte $70,$4D,$00,$10,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
  .byte $0D,$0D,$0D,$0D,$0D,$0D,$8D,$02
  .byte $41,<dlist1,>dlist1

dlist2:
  .byte $70,$70,$70,$42,$00
  .byte $10,$02,$02,$02,$02,$02,$02,$02
  .byte $02,$02,$02,$02,$02,$02,$02,$02
  .byte $02,$02,$02,$02,$02,$02,$02,$02
  .byte $41,<dlist2,>dlist2

; *****************************************************************************
; * Program entry point                                                       *
; *****************************************************************************
.proc main
  sei
  lda #$00
  sta NMIEN
  sta DMACTL
  sta IRQEN

  ldx #$FF
  txs
  cld

  jsr reset_hardware
  jsr clear_memory

  jsr init_trak_ball
  jsr configure_system

  ; enable vbi
  lda #$40
  sta NMIEN

.ifdef DEBUG
  jsr debug
.endif

  ; pass control to the game
  jmp run
.endproc

.ifdef DEBUG
; *****************************************************************************
; * Display trak-ball readings                                                *
; *****************************************************************************
.proc debug
  lda #$0E
  sta PCOLR0+5
  lda #$84
  sta PCOLR0+6
  
  ; display centre positions read with POTs disabled
  ldy #0
  lda ball_cx
  jsr print_hex
  
  ldy #3
  lda ball_cy
  jsr print_hex
  
forever:
  ; display current positions
  ldy #40
  lda pot2
  jsr print_hex
  
  ldy #43
  lda pot3
  jsr print_hex
  
  jmp forever
.endproc

; *****************************************************************************
; * Display hex value in A at Y offset in debug screen memory                 *
; *****************************************************************************
.proc print_hex
  pha
  
  lsr
  lsr
  lsr
  lsr
  tax
  lda digits,x
  sta DEBUG_SCREEN_MEMORY,y
  
  pla
  
  and #$0F
  tax
  lda digits,x
  sta DEBUG_SCREEN_MEMORY+1,y
  
  rts
  
digits:
  scrcode "0123456789ABCDEF"
.endproc
.endif

; *****************************************************************************
; * Configure system                                                          *
; *****************************************************************************
.proc configure_system
  lda #<blank_dlist
  sta SDLSTL
  lda #>blank_dlist
  sta SDLSTH

  lda #$22
  sta SDMCTL

  lda #<vbi
  sta VVBLKI
  lda #>vbi
  sta VVBLKI+1

  lda #<XITVBV
  sta VVBLKD
  lda #>XITVBV
  sta VVBLKD+1

  lda #$0F
  sta PORTA

  lda #$07
  sta consol
  
  lda KBCODE
  sta kbcode

  lda #$00
  sta pending_count

  lda #$F8
  sta CHBASE

  lda #$80
  sta crosshair_x
  lda #$40
  sta crosshair_y
  lda #$00
  sta erase_y
  
  rts
.endproc

; *****************************************************************************
; * Frame delay                                                               *
; *****************************************************************************
.proc frame_delay
:
  lda VCOUNT
  bne :-
  
:
  lda VCOUNT
  beq :-
  
  rts
.endproc

; *****************************************************************************
; * Set deferred vertical blank vector                                        *
; *****************************************************************************
.proc SETVBV
  txa
  ldx #$05
  sta WSYNC
:
  dex
  bne :-
  sty VVBLKD
  sta VVBLKD+1
  rts
.endproc

; *****************************************************************************
; * Exit vertical blank routine                                               *
; *****************************************************************************
.proc XITVBV ; exit vertical blank
  pla
  tay
  pla
  tax
  pla
  rti
.endproc

; *****************************************************************************
; * Vertical blank interrupt routine                                          *
; *****************************************************************************
.proc vbi
  pha
  txa
  pha
  tya
  pha

  ; copy shadow DMA control to h/w
  lda SDMCTL
  sta DMACTL

  ; copy shadow display list to h/w
  lda SDLSTL
  sta DLISTL
  lda SDLSTH
  sta DLISTH

  ; copy shadow PRIOR to h/w
  lda GPRIOR
  sta PRIOR
  
  ; copy shadow colour registers to h/w
  ldx #$08
:
  lda PCOLR0,x
  sta COLPM0,x
  dex
  bpl :-

  jsr read_trak_ball
  jsr read_consol  
  jsr display_crosshair

  jmp (VVBLKD)
.endproc

; *****************************************************************************
; * Initialise trak-ball reading centre position                              *
; *****************************************************************************
.proc init_trak_ball
  ; select port 2, disable POTs
  lda #$01
  sta CONSOL
  
  jsr frame_delay
  
  ; start POT read
  sta POTGO

  jsr frame_delay

  ; read centre positions
  lda POT2
  sta ball_cx
  lda POT3
  sta ball_cy

  ; select port 2, enable POTs
  lda #$05
  sta CONSOL

  rts
.endproc

; *****************************************************************************
; * Read trak-ball                                                            *
; *****************************************************************************
.proc read_trak_ball
  ldx #$00
  jsr process_pot

  ldx #$01
  jsr process_pot

  lda SKSTAT
  lsr
  lsr
  lsr
  and TRIG1
  and #$01
  tax
  lda porta_values,x
  sta PORTA

  sta POTGO

  rts

porta_values:
  .byte $0F,$0E
.endproc

; *****************************************************************************
; * Process trak-ball pot indexed by X                                        *
; *****************************************************************************
.proc process_pot
  ; read pot and substract centre value
  lda POT2,x
  sta pot2,x
  sec
  sbc ball_cx,x

  ; round up by 2 before dividing by 4
  clc
  adc #$02

  ; divide by 4
  cmp #$80 ; puts sign in C
  php      ; signed shift right twice
  ror
  plp
  ror
  
  ; update crosshair coord
  clc
  adc crosshair_x,x
  cmp low_limit,x
  bcc too_low
  cmp high_limit,x
  bcs too_high
  sta crosshair_x,x
  rts  

too_low:
  lda low_limit,x
  sta crosshair_x,x
  rts

too_high:
  lda high_limit,x
  sta crosshair_x,x
  rts
  
low_limit:
  .byte $2D,$10
  
high_limit:
  .byte $CC,$6F
  
.endproc

; *****************************************************************************
; * Read keypad and make it look like the CONSOL keys                         *
; *****************************************************************************
.proc read_consol
  lda #$07
  sta consol
  
  lda KBCODE
  cmp kbcode
  bne key_pressed

  lda pending_count
  bne debounce
  rts
  
debounce:
  dec pending_count
  beq debounced
  rts
  
debounced:
  lda pending_key
  cmp #$42
  beq reset
  sta consol
  rts
  
reset:
  jmp main
  
key_pressed:
  sta kbcode
  lsr
  and #$0F

  ldx #$03
:
  cmp keypad_code,x
  beq found_key
  dex
  bpl :-
  rts

found_key:
  lda consol_code,x
  sta pending_key
  lda #$08
  sta pending_count
  rts

keypad_code:
  .byte $0C,$08,$03,$04 ; start, pause, *, reset

consol_code:
  .byte $06,$03,$05,$42 ; start, option, select

.endproc

; *****************************************************************************
; * Clear RAM from $200..$3FFF                                                *
; *****************************************************************************
.proc clear_memory
  lda #$00
  sta zp_temp
  lda #$02
  sta zp_temp+1

  lda #$00
  tay
:
  sta (zp_temp),y
  iny
  bne :-
  inc zp_temp+1
  ldx zp_temp+1
  cpx #$40
  bne :-
  rts
.endproc

; *****************************************************************************
; * Reset hardware                                                            *
; *****************************************************************************
.proc reset_hardware
  lda #$00
  ldx #$1F
:
  sta HPOSP0,x
  sta AUDF1,x
  sta DMACTL,x
  dex
  bpl :-

  lda #$02
  sta SKCTL

  rts
.endproc

; *****************************************************************************
; * Blank display list                                                        *
; *****************************************************************************
.proc blank_dlist
.ifdef DEBUG
  .byte $70,$70,$70
  .byte $42
  .word DEBUG_SCREEN_MEMORY
  .byte $02
.endif
  .byte $41,<blank_dlist,>blank_dlist
.endproc

; *****************************************************************************
; * Bank switch according to A                                                *
; *****************************************************************************
.proc select_bank_a
  sta zp_temp
  stx zp_temp2
  and #$07
  tax
  lda BANK_SELECTOR,x
  lda zp_temp
  ldx zp_temp2
  rts
.endproc

; *****************************************************************************
; * Display crosshair                                                         *
; *****************************************************************************
.proc display_crosshair
  ; erase old image
  ldy erase_y
  lda #$00
  sta PLAYER_1_MEMORY-3,y
  sta PLAYER_1_MEMORY-2,y
  sta PLAYER_1_MEMORY,y
  sta PLAYER_1_MEMORY+2,y
  sta PLAYER_1_MEMORY+3,y

  ; set horizontal position
  lda crosshair_x
  sta HPOSP1
  sec
  sbc #$2D
  sta penh

  lda crosshair_y
  sta erase_y
  tay
  clc
  adc #$02
  sta penv

  ; draw new image
  lda #$10
  sta PLAYER_1_MEMORY-3,y
  sta PLAYER_1_MEMORY-2,y
  sta PLAYER_1_MEMORY+2,y
  sta PLAYER_1_MEMORY+3,y
  lda #$C6
  sta PLAYER_1_MEMORY,y
  
  rts
.endproc

.segment "CODE"

POKMSK      = $0010
CRITIC      = $0042
ATRACT      = $004D
LOMEM       = $0080

VDSLST      = $0206
SDMCTL      = $022F
SDLSTL      = $0230
SDLSTH      = $0231
GPRIOR      = $026F
PCOLR0      = $02C0
COLOR0      = $02C4
COLOR1      = $02C5
COLOR2      = $02C6
COLOR3      = $02C7
COLOR4      = $02C8

HPOSP0      = $C000
COLPF0      = $C016
COLPF1      = $C017
COLPF2      = $C018
GRACTL      = $C01D

AUDF1       = $E800
RANDOM      = $E80A
IRQEN       = $E80E
SKCTL       = $E80F

PMBASE      = $D407
WSYNC       = $D40A
VCOUNT      = $D40B
NMIEN       = $D40E

;
; Code equates
;
L0082       = $0082
L0083       = $0083
L0084       = $0084
L0085       = $0085
L0086       = $0086
L0087       = $0087
L0088       = $0088
L0089       = $0089
L008A       = $008A
L008B       = $008B
L008C       = $008C
L008D       = $008D
L008E       = $008E
L008F       = $008F
L0090       = $0090
L0091       = $0091
L0092       = $0092
L0093       = $0093
L0094       = $0094
L0095       = $0095
L0096       = $0096
L0097       = $0097
L0098       = $0098
L0099       = $0099
L009A       = $009A
L009B       = $009B
L009C       = $009C
L009D       = $009D
L009E       = $009E
L009F       = $009F
L00A0       = $00A0
L00A1       = $00A1
L00A2       = $00A2
L00A3       = $00A3
L00A4       = $00A4
L00A5       = $00A5
L00A6       = $00A6
L00A7       = $00A7
L00A8       = $00A8
L00A9       = $00A9
L00AB       = $00AB
L00AC       = $00AC
L00AD       = $00AD
L00C0       = $00C0
L00C1       = $00C1
L00C2       = $00C2
L00C6       = $00C6
L00C7       = $00C7
L00C8       = $00C8
L00C9       = $00C9
L00CA       = $00CA
L00CB       = $00CB
L00CC       = $00CC
L00CD       = $00CD
L0480       = $0480
L0488       = $0488
L0490       = $0490
L0498       = $0498
L0499       = $0499
L04A0       = $04A0
L04A1       = $04A1
L04A8       = $04A8
L04A9       = $04A9
L0A00       = $0A00
L0E00       = $0E00
L0F00       = $0F00
L1E60       = $1E60
LD500       = $D500

;
; Start of code
;
;            *= $A000
;
            .byte $9A,$96,$55,$56,$66,$65,$13,$05
            .byte $55,$56,$65,$13,$05,$55,$56,$65
            .byte $13,$05,$55,$56,$65,$13,$05,$55
            .byte $56,$66,$A5,$A9,$9A,$96,$55,$56
            .byte $66,$65,$13,$05,$55,$56,$65,$13
            .byte $05,$55,$56,$65,$13,$05,$55,$56
            .byte $65,$13,$05,$55,$56,$66,$25,$89
            .byte $9A,$96,$40,$06,$66,$6A,$13,$06
            .byte $AA,$6A,$13,$06,$AA,$6A,$13,$06
            .byte $AA,$6A,$13,$06,$AA,$66,$25,$89
            .byte $9A,$96,$4E,$C6,$46,$67,$00,$35
            .byte $70,$03,$57,$00,$37,$67,$00,$35
            .byte $70,$03,$57,$00,$37,$67,$00,$35
            .byte $70,$03,$57,$00,$37,$67,$00,$35
            .byte $70,$03,$57,$00,$37,$6A,$A5,$AA
            .byte $9A,$96,$0E,$CA,$76,$64,$00,$05
            .byte $40,$00,$54,$00,$04,$64,$00,$05
            .byte $40,$00,$54,$00,$04,$64,$00,$05
            .byte $40,$00,$54,$00,$04,$64,$00,$05
            .byte $40,$00,$54,$00,$04,$40,$00,$00
            .byte $9A,$96,$40,$06,$56,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$43,$F0,$FC
            .byte $9A,$96,$00,$02,$56,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$40,$00,$00
            .byte $9A,$96,$4E,$C6,$56,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$66,$59,$99
            .byte $9A,$96,$4E,$C6,$56,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$66,$59,$99
            .byte $9A,$96,$40,$06,$56,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$66,$59,$99
            .byte $9A,$96,$55,$56,$56,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$66,$59,$99
            .byte $6A,$96,$55,$56,$56,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$66,$59,$96
            .byte $5A,$96,$55,$56,$56,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$66,$59,$65
            .byte $9A,$96,$55,$56,$A6,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$66,$56,$59
            .byte $6A,$96,$55,$55,$66,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$64,$00,$05
            .byte $40,$00,$54,$00,$06,$65,$95,$96
            .byte $5A,$96,$AA,$AA,$66,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$67,$00,$35
            .byte $70,$03,$57,$00,$36,$13,$02,$65
            .byte $9A,$95,$55,$56,$66,$65,$13,$05
            .byte $55,$56,$65,$13,$05,$55,$56,$65
            .byte $13,$05,$55,$56,$65,$13,$05,$55
            .byte $56,$65,$5A,$59,$6A,$AA,$AA,$A6
            .byte $66,$65,$13,$05,$55,$56,$65,$13
            .byte $05,$55,$56,$65,$13,$05,$55,$56
            .byte $65,$13,$05,$55,$56,$65,$00,$06
            .byte $5A,$00,$00,$66,$66,$65,$13,$05
            .byte $55,$56,$65,$13,$05,$55,$56,$65
            .byte $13,$05,$55,$56,$65,$13,$05,$55
            .byte $56,$60,$BF,$E0,$9A,$00,$00,$66
            .byte $66,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$65,$00,$05,$9A,$55,$54,$66
            .byte $66,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$65,$5A,$55,$9A,$FF,$FF,$66
            .byte $66,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$65,$00,$05,$9A,$55,$54,$66
            .byte $66,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$60,$BF,$E0,$9A,$FF,$FF,$66
            .byte $66,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$65,$00,$05,$9A,$55,$54,$66
            .byte $66,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$65,$5A,$55,$9A,$FF,$FF,$66
            .byte $66,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$65,$00,$05,$9A,$55,$54,$66
            .byte $66,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$60,$BF,$E0,$9A,$FF,$FF,$66
            .byte $66,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$65,$00,$05,$9A,$55,$54,$66
            .byte $66,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$65,$5A,$AA,$9A,$FF,$FF,$66
            .byte $66,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$7A,$99,$9D,$9A,$55,$54,$66
            .byte $66,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$64,$00,$05,$40,$00,$54,$00
            .byte $06,$6A,$99,$9D,$9A,$FF,$FF,$66
            .byte $66,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$67,$00,$35,$70,$03,$57,$00
            .byte $36,$65,$99,$9D,$9A,$55,$54,$66
            .byte $66,$65,$13,$05,$55,$56,$65,$13
            .byte $05,$55,$56,$65,$13,$05,$55,$56
            .byte $65,$13,$05,$55,$56,$65,$99,$9D
            .byte $9A,$FF,$FF,$A6,$66,$65,$13,$05
            .byte $55,$56,$65,$13,$05,$55,$56,$65
            .byte $13,$05,$55,$56,$65,$13,$05,$55
            .byte $56,$65,$99,$9D,$9A,$55,$54,$16
            .byte $66,$6A,$AA,$AA,$A5,$13,$03,$AA
            .byte $6A,$13,$06,$AA,$65,$13,$05,$55
            .byte $56,$65,$13,$05,$55,$56,$6A,$99
            .byte $9D,$9A,$FF,$FF,$56,$66,$13,$02
            .byte $55,$65,$95,$13,$02,$55,$65,$55
            .byte $55,$59,$13,$03,$55,$6A,$AA,$AA
            .byte $A6,$13,$03,$AA,$66,$13,$06,$AA
            .byte $69,$59,$9D,$9A,$55,$54,$16,$66
            .byte $72,$AA,$AA,$A5,$95,$55,$55,$5A
            .byte $A5,$55,$55,$59,$13,$03,$AA,$A5
            .byte $65,$65,$66,$56,$56,$55,$55,$66
            .byte $13,$06,$55,$69,$A9,$9D,$9A,$FF
            .byte $FF,$6A,$66,$55,$55,$95,$55,$13
            .byte $02,$CC,$D9,$13,$02,$CC,$D9,$13
            .byte $02,$CC,$EA,$A5,$73,$13,$05,$33
            .byte $67,$13,$06,$33,$69,$99,$AD,$9A
            .byte $55,$54,$25,$66,$55,$55,$65,$55
            .byte $13,$02,$00,$19,$13,$02,$00,$19
            .byte $13,$02,$00,$16,$65,$40,$13,$05
            .byte $00,$64,$13,$06,$00,$69,$99,$55
            .byte $9A,$FF,$FF,$66,$A6,$55,$55,$59
            .byte $55,$13,$02,$00,$19,$13,$02,$00
            .byte $19,$13,$02,$00,$16,$69,$40,$13
            .byte $05,$00,$64,$00,$00,$0F,$FF,$13
            .byte $02,$00,$7D,$99,$56,$5A,$55,$54
            .byte $26,$66,$AA,$AA,$96,$55,$13,$02
            .byte $CC,$D9,$13,$02,$CC,$D9,$13,$02
            .byte $CC,$D6,$66,$40,$13,$05,$00,$64
            .byte $00,$00,$0F,$FF,$13,$02,$00,$7D
            .byte $99,$59,$66,$FF,$FF,$66,$65,$55
            .byte $55,$65,$95,$13,$02,$55,$59,$13
            .byte $02,$55,$59,$13,$02,$55,$56,$66
            .byte $40,$13,$05,$00,$64,$00,$00,$0F
            .byte $FF,$13,$02,$00,$45,$9A,$A5,$96
            .byte $55,$54,$26,$65,$55,$AA,$59,$65
            .byte $13,$02,$55,$59,$13,$02,$55,$59
            .byte $13,$02,$55,$56,$66,$40,$13,$05
            .byte $00,$64,$00,$00,$0F,$FF,$13,$02
            .byte $00,$AA,$99,$56,$56,$FF,$FF,$A6
            .byte $65,$55,$95,$96,$5A,$13,$02,$CC
            .byte $D9,$13,$02,$CC,$D9,$13,$02,$CC
            .byte $D6,$66,$40,$13,$05,$00,$64,$13
            .byte $06,$00,$45,$9A,$A9,$56,$55,$54
            .byte $16,$6A,$B1,$95,$65,$95,$13,$02
            .byte $00,$19,$13,$02,$00,$19,$13,$02
            .byte $00,$16,$66,$73,$13,$05,$33,$6B
            .byte $13,$06,$33,$AA,$99,$55,$56,$FF
            .byte $FF,$56,$65,$55,$95,$59,$65,$13
            .byte $02,$00,$19,$13,$02,$00,$19,$13
            .byte $02,$00,$16,$66,$65,$55,$55,$66
            .byte $55,$56,$66,$13,$07,$55,$45,$5A
            .byte $AA,$F6,$55,$54,$16,$6A,$AA,$95
            .byte $56,$5A,$13,$02,$CC,$E9,$13,$02
            .byte $CC,$E9,$13,$02,$CC,$EA,$66,$13
            .byte $05,$AA,$A6,$13,$0A,$AA,$F6,$FF
            .byte $FF,$AA,$65,$13,$02,$55,$95,$13
            .byte $0B,$55,$66,$13,$11,$55,$56,$55
            .byte $54,$15,$6A,$13,$23,$AA,$13,$FF
run:
            sei
            cld
            lda #$00
            ldx #$7F
LA6C6:      sta LOMEM,X
            dex
            bpl LA6C6
            txs
            lda #$2E
            sta SDMCTL
            lda #$03
            sta GRACTL
            lda #$02
            sta GPRIOR
            lda #$08
            sta PMBASE
            lda #$02
            sta SKCTL
            lda #$C1
            sta VDSLST
            lda #$AF
            sta VDSLST+1
            lda #$07
            ldx #$AF
            ldy #$D5
            jsr SETVBV
            lda #$C0
            sta NMIEN
            lda #$00 ; $40 no interrupts
            sta POKMSK
            sta IRQEN
            cli
            lda #$00
            ldx #$50 ; $90 bank now starts at $4000 not $8000
            ldy #$20
            jsr LAF56
            lda #$CD
            ldx #$5A ; $9A bank now starts at $4000 not $8000
            ldy #$30
            jsr LAF56
            lda #$00
            ldx #$10
            ldy #$00
LA71D:      sta L0E00,Y
            txa
            sta L0F00,Y
            lda L0E00,Y
            clc
            adc #$28
            bcc LA72D
            inx
LA72D:      iny
            cpy #$5F
            bne LA71D
            jsr LA774
            jsr LAE39
LA738:      jsr LA75A
LA73B:      lda L00A4
            beq LA74C
            lda #$00
            ldy #$87
LA743:      .byte $99,$00,$00      ; sta $0000,X
            iny
            bne LA743
            jsr LAACB
LA74C:      lda L00A3
            bne LA73B
LA750:      lda L0083
            beq LA750
            lda #$00
            sta L0083
            beq LA738
LA75A:      lda L0090
            and #$06
            tax
            lda LA76C,X
            sta L00A7
            lda LA76D,X
            sta L00A8
            jmp (L00A7)
LA76C:      .byte $A7
LA76D:      .byte $A7,$71,$AB,$9A,$A9,$DC,$A7
LA774:      lda #$00
            ldx #$20
            jsr LAE9D
            ldx #$00
            ldy #$5C
            lda #$4F
            sta L00A7
            lda #$B0
            sta L00A8
            jsr LAE72
            lda #$00
            ldy #$86
LA78E:      .byte $99,$00,$00      ; sta $0000,X
            iny
            bne LA78E
            lda #$3D
            ldy #$56
            ldx #$00
            jsr LAFA7
            lda #$3D
            ldy #$56
            ldx #$02
            jsr LAFA7
            rts
            jsr LACE8
            lda L0091
            beq LA7DB
            lda #$00
            sta ATRACT
            sta L0091
            lda L008A
            cmp #$4C
            bcc LA7DB
            cmp #$54
            bcs LA7DB
            lda L0089
            cmp #$05
            bcc LA7CC
            cmp #$37
            bcs LA7CC
            lda #$00
            beq LA7D6
LA7CC:      cmp #$69
            bcc LA7DB
            cmp #$9B
            bcs LA7DB
            lda #$01
LA7D6:      sta L0086
            jsr LAACB
LA7DB:      rts
            lda L0095
            beq LA7E3
            dec L0095
            rts
LA7E3:      lda #<dlist2
            sta SDLSTL
            lda #>dlist2
            sta SDLSTH
            lda #$C0
            sta COLOR2
            lda #$0C
            sta COLOR1
            jsr LAF3E
            ldx L009D
            inx
            txa
            ldx #$00
            jsr LAA26
            jsr LA9ED
            ldx #$1B
            ldy #$01
            jsr LAE6A
            ldx #$05
            ldy #$01
            lda L009E
            bne LA81F
            lda #$E1
            sta L00A7
            lda #$B0
            sta L00A8
            bne LA827
LA81F:      lda #$FC
            sta L00A7
            lda #$B0
            sta L00A8
LA827:      jsr LAE72
            ldx #$03
            ldy #$05
            lda #$78
            sta L00A7
            lda #$B0
            sta L00A8
            jsr LAE72
            lda L0091
            ldx L0092
            jsr LAA26
            jsr LA9ED
            ldx #$1E
            ldy #$05
            jsr LAE6A
            ldx #$03
            ldy #$07
            lda #$8F
            sta L00A7
            lda #$B0
            sta L00A8
            jsr LAE72
            lda L0093
            ldx #$00
            jsr LAA26
            jsr LA9ED
            ldx #$1E
            ldy #$07
            jsr LAE6A
            ldx #$03
            ldy #$09
            lda #$A6
            sta L00A7
            lda #$B0
            sta L00A8
            jsr LAE72
            lda L0093
            ldx #$00
            ldy #$64
            jsr LAA6C
            sta L00A7
            stx L00A8
            lda L0091
            ldy L0092
            jsr LAA96
            sta L0091
            ldx #$00
            jsr LAA26
            jsr LA9ED
            ldx #$1E
            ldy #$09
            jsr LAE6A
            ldx #$03
            ldy #$0C
            lda #$B8
            sta L00A7
            lda #$B0
            sta L00A8
            jsr LAE72
            lda L0097
            ldx L0098
            ldy L0099
            jsr LA9ED
            ldx #$1E
            ldy #$0C
            jsr LAE6A
            ldx #$03
            ldy #$0E
            lda #$C4
            sta L00A7
            lda #$B0
            sta L00A8
            jsr LAE72
            lda L0093
            asl
            adc L0093
            ldx #$00
            ldy L0091
            jsr LAA6C
            jsr LAA26
            pha
            sed
            clc
            adc L0097
            sta L0097
            txa
            adc L0098
            sta L0098
            tya
            adc L0099
            sta L0099
            cld
            pla
            jsr LA9ED
            ldx #$1E
            ldy #$0E
            jsr LAE6A
            ldx #$03
            ldy #$10
            lda #$D4
            sta L00A7
            lda #$B0
            sta L00A8
            jsr LAE72
            lda L0097
            ldx L0098
            ldy L0099
            jsr LA9ED
            ldx #$1E
            ldy #$10
            jsr LAE6A
            ldx L009D
            cpx #$05
            bcc LA91F
            ldx #$05
LA91F:      lda LB049,X
            cmp L0091
            bcc LA980
            ldx L009E
            inx
            txa
            ora L009F
            sta L009F
            lda L0099
            cmp L0082
            bcc LA950
            bne LA944
            lda L0098
            cmp LOMEM+1
            bcc LA950
            bne LA944
            lda L0097
            cmp LOMEM
            bcc LA950
LA944:      lda L0097
            sta LOMEM
            lda L0098
            sta LOMEM+1
            lda L0099
            sta L0082
LA950:      ldx #$05
            ldy #$12
            lda #$31
            sta L00A7
            lda #$B1
            sta L00A8
            jsr LAE72
            lda L0099
            and #$F0
            beq LA969
            lda #$0A
            bne LA96D
LA969:      lda L0099
            and #$0F
LA96D:      asl
            tax
            lda LB1B7,X
            sta L00A7
            lda LB1B8,X
            sta L00A8
            ldx #$0F
            ldy #$13
            jsr LAE72
LA980:      ldx #$07
            ldy #$17
            lda #$17
            sta L00A7
            lda #$B1
            sta L00A8
            jsr LAE72
            sta L0095
            lda #$05
            sta L0096
            lda #$04
            sta L0090
            rts
            lda L0095
            bne LA9A4
            lda L0096
            beq LA9AC
            dec L0096
LA9A4:      dec L0095
            jsr LAE29
            bcs LA9AC
            rts
LA9AC:      ldx L0086
            lda LA9EB,X
            eor L009F
            beq LA9E8
            txa
            beq LA9E3
            lda L009E
            eor #$01
            clc
            adc #$01
            eor L009F
            beq LA9E3
            ldx L0097
            lda L009A
            stx L009A
            sta L0097
            ldx L0098
            lda L009B
            stx L009B
            sta L0098
            ldx L0099
            lda L009C
            stx L009C
            sta L0099
            lda L009E
            eor #$01
            sta L009E
            bne LA9E5
LA9E3:      inc L009D
LA9E5:      jmp LAACB
LA9E8:      jmp LA774
LA9EB:      .byte $01,$03
LA9ED:      pha
            txa
            pha
            tya
            pha
            ldx #$00
            ldy #$00
            lda #$06
            .byte $99,$AD,$00      ; sta L00AD,X
            iny
LA9FC:      pla
            pha
            lsr
            lsr
            lsr
            lsr
            jsr LAA0E
            pla
            jsr LAA0E
            cpy #$07
            bcc LA9FC
            rts
LAA0E:      and #$0F
            bne LAA1E
            cpx #$00
            bne LAA1F
            cpy #$06
            beq LAA1F
            lda #$20
            bne LAA21
LAA1E:      inx
LAA1F:      ora #$30
LAA21:      .byte $99,$AD,$00      ; sta L00AD,X
            iny
            rts
LAA26:      sed
            sta L00A5
            stx L00A6
            lda #$01
            sta L00A9
            lda #$00
            sta L00A7
            sta L00A8
            tax
            tay
LAA37:      lsr L00A6
            bne LAA43
            ror L00A5
            bcs LAA47
            bne LAA54
            cld
            rts
LAA43:      ror L00A5
            bcc LAA54
LAA47:      clc
            adc L00A9
            pha
            txa
            adc L00A7
            tax
            tya
            adc L00A8
            tay
            pla
LAA54:      pha
            clc
            lda L00A9
            adc L00A9
            sta L00A9
            lda L00A7
            adc L00A7
            sta L00A7
            lda L00A8
            adc L00A8
            sta L00A8
            pla
            jmp LAA37
LAA6C:      sta L00A5
            stx L00A6
            sty L00A8
            lda #$00
            sta L00A7
            ldy #$08
LAA78:      lsr
            ror L00A7
            ror L00A8
            bcc LAA8A
            tax
            clc
            lda L00A7
            adc L00A5
            sta L00A7
            txa
            adc L00A6
LAA8A:      dey
            bne LAA78
            lsr
            lda L00A7
            ror
            tax
            lda L00A8
            ror
            rts
LAA96:      cmp #$00
            bne LAA9E
            cpy #$00
            beq LAAC8
LAA9E:      sta L00A5
            sty L00A6
            lda #$00
            sta L00A9
            ldy #$08
LAAA8:      asl L00A7
            rol L00A8
            rol L00A9
            lda L00A8
            cmp L00A5
            lda L00A9
            sbc L00A6
            bcc LAAC2
            sta L00A9
            lda L00A8
            sbc L00A5
            sta L00A8
            inc L00A7
LAAC2:      dey
            bne LAAA8
            lda L00A7
            rts
LAAC8:      lda #$00
            rts
LAACB:      lda #$00
            ldx #$30
            jsr LAE9D
            jsr LAE39
            jsr LAAF7
            jsr LAB59
            lda #$00
            sta L0091
            sta L0092
            sta L0093
            sta L008F
            lda L009D
            cmp #$05
            bcc LAAED
            lda #$05
LAAED:      asl
            adc #$0E
            sta L0094
            lda #$02
            sta L0090
            rts
LAAF7:      lda #$00
            ldy #$27
LAAFB:      sta L1E60,Y
            dey
            bpl LAAFB
            lda L0097
            ldx L0098
            ldy L0099
            jsr LA9ED
            ldy #$5C
            ldx #$03
            lda L009E
            beq LAB14
            ldx #$1C
LAB14:      jsr LAE6A
            lda L0086
            beq LAB31
            lda L009A
            ldx L009B
            ldy L009C
            jsr LA9ED
            ldy #$5C
            ldx #$1C
            lda L009E
            beq LAB2E
            ldx #$03
LAB2E:      jsr LAE6A
LAB31:      lda LOMEM
            ldx LOMEM+1
            ldy L0082
            jsr LA9ED
            ldy #$5C
            ldx #$0F
            jsr LAE6A
            lda #$60
            sta L00A7
            lda #$1E
            sta L00A8
            lda #$5E
            clc
            adc L009E
            ldy #$09
            bit L009E
            beq LAB56
            ldy #$1B
LAB56:      sta (L00A7),Y
            rts
LAB59:      lda #$00
LAB5B:      tax
            lda #$00
            sta L00C6,X
            lda #$00
            sta L00CC,X
            lda #$FF
            sta L00CB,X
            txa
            clc
            adc #$0E
            cmp #$38
            bcc LAB5B
            rts
            lda #$00
            sta ATRACT
            jsr LACE8
            lda L008B
            clc
            adc #$0E
            cmp #$38
            bcc LAB83
            lda #$00
LAB83:      sta L008B
            tax
            ldy L00C6,X
            beq LABAA
            lda L00C7,X
            beq LAB91
            dec L00C7,X
            rts
LAB91:      lda L00C0,X
            sta L008C
            lda L00C1,X
            sta L008D
            lda LBE25,Y
            sta L00A7
            lda LBE26,Y
            sta L00A8
            ldy #$00
            sty L008E
            jmp (L00A7)
LABAA:      lda L0094
            bne LABCC
            lda #$00
LABB0:      tay
            .byte $B9,$C6,$00      ; lda L00C6,X
            bne LABCB
            tya
            clc
            adc #$0E
            cmp #$38
            bcc LABB0
            lda #$1E
            sta L0095
            lda #$00
            sta HPOSP0
            lda #$06
            sta L0090
LABCB:      rts
LABCC:      dec L0094
            stx L00A9
            lda RANDOM
            and #$07
            tax
            lda LB25F,X
            pha
            lda LB267,X
            tay
            ldx #$04
            inc L00A2
            lda L00A2
            and #$01
            beq LABEA
            ldx #$06
LABEA:      pla
            jsr LAFA7
            ldx L00A9
            ldy RANDOM
            bit RANDOM
LABF6:      tya
            and #$07
            tay
            lda LBE0D,Y
            and L008F
            beq LAC09
            bvs LAC06
            dey
            bvc LABF6
LAC06:      iny
            bvs LABF6
LAC09:      lda L00CB,X
            and L008F
            ora LBE0D,Y
            sta L008F
            lda LBE0D,Y
            eor #$FF
            sta L00CB,X
            lda LBE15,Y
            sta L00C8,X
            lda LBE1D,Y
            sta L00C9,X
            lda RANDOM
            and #$1F
            sta L00C7,X
            lda RANDOM
            and #$1E
            sta L00A9
            lda L009D
            cmp #$05
            bcc LAC39
            lda #$05
LAC39:      asl
            asl
            asl
            asl
            asl
            ora L00A9
            tay
            lda LB418,Y
            sta L008C
            lda LB419,Y
            sta L008D
            lda #$00
            sta L008E
            jmp LAC53
            rts
LAC53:      ldx L008B
LAC55:      ldy L008E
LAC57:      lda (L008C),Y
            iny
            sta L00A9
            bit L00A9
            bmi LAC78
            bvs LAC75
            and #$3F
            sta L00C6,X
            tya
            clc
            adc L008C
            bcc LAC6E
            inc L008D
LAC6E:      sta L00C0,X
            lda L008D
            sta L00C1,X
            rts
LAC75:      jmp LAC57
LAC78:      bvs LAC8C
            and #$3F
            stx L00A9
            clc
            adc L00A9
            tax
            lda (L008C),Y
            iny
            sta L00C2,X
            ldx L00A9
            jmp LAC57
LAC8C:      and #$3F
            stx L00A9
            clc
            adc L00A9
            tax
            lda (L008C),Y
            dec L00C2,X
            bne LACA0
            ldx L00A9
            iny
            jmp LAC57
LACA0:      ldx L00A9
            sty L00A9
            sec
            sbc L00A9
            sta L00A9
            clc
            lda L008C
            sbc L00A9
            sta L008C
            bcs LACB4
            dec L008D
LACB4:      ldy #$00
            jmp LAC57
            ldy L00CA,X
            iny
            jmp LACC2
            ldy L00CA,X
            dey
LACC2:      tya
            sta L00CA,X
            sta L00A9
            ldy L00C9,X
            lda L00C8,X
            tax
            lda L00A9
            jsr LAEE3
            jmp LAC53
            lda (L008C),Y
            iny
            sty L008E
            sta L00CA,X
            pha
            ldy L00C9,X
            lda L00C8,X
            tax
            pla
            jsr LAEE3
            jmp LAC53
LACE8:      jsr LAE29
            bcs LACF0
            jmp LAE02
LACF0:      lda #$01
            ldy #$06
            ldx #$00
            jsr LAFA7
            lda #$0B
            ldy #$10
            ldx #$02
            jsr LAFA7
            inc L0091
            bne LAD08
            inc L0092
LAD08:      lda #$0E
            ldx #$04
LAD0C:      sta COLPF0,X
            ldy COLOR0,X
            sta COLOR0,X
            sty L00AD,X
            dex
            bpl LAD0C
            stx CRITIC
            lda VCOUNT
LAD1F:      cmp VCOUNT
            beq LAD1F
LAD24:      cmp VCOUNT
            bne LAD24
            lda #$00
            sta CRITIC
            ldy penv
; here
            lda penh
            nop ; sec
            nop ; sbc #$5C
            nop
            nop ; bcs LAD3A
            nop
            nop ; adc #$E3
            nop
LAD3A:      sta L0089
            tya
            sec
            sbc #$12
            sta L008A
            ldx #$04
LAD44:      lda L00AD,X
            sta COLPF0,X
            sta COLOR0,X
            dex
            bpl LAD44
            ldx #$00
LAD51:      lda L00C6,X
            beq LAD59
            lda L00CC,X
            bne LAD65
LAD59:      txa
            clc
            adc #$0E
            tax
            cmp #$38
            bcc LAD51
            jmp LADDC
LAD65:      lda L00CA,X
            tay
            lda L00C8,X
            clc
            adc LB278,Y
            sec
            sbc L0089
            bcs LAD77
            eor #$FF
            adc #$01
LAD77:      cmp #$07
            bcs LAD59
            lda L00C9,X
            adc LB348,Y
            sec
            sbc L008A
            bcs LAD89
            eor #$FF
            adc #$01
LAD89:      cmp #$07
            bcs LAD59
            inc L0093
            sed
            clc
            lda L00CD,X
            adc L0098
            sta L0098
            lda L0099
            adc #$00
            sta L0099
            tay
            cld
            txa
            pha
            lda L0097
            ldx L0098
            jsr LA9ED
            ldx #$03
            lda L009E
            beq LADB0
            ldx #$1C
LADB0:      ldy #$5C
            jsr LAE6A
            lda #$15
            ldy #$1F
            ldx #$00
            jsr LAFA7
            lda #$1A
            ldy #$1F
            ldx #$02
            jsr LAFA7
            pla
            tax
            lda #$00
            sta L00C7,X
            sta L00CC,X
            sta L008E
            lda #$6F
            sta L008C
            lda #$B2
            sta L008D
            jsr LAC55
LADDC:      lda L0087
            beq LADE9
            lda #$00
            tay
LADE3:      sta L0A00,Y
            iny
            bpl LADE3
LADE9:      lda L0089
            clc
            adc #$2C
            bcc LADF2
            lda #$00
LADF2:      sta HPOSP0
            lda L008A
            clc
            adc #$0C
            sta L0088
            ldx #$00
            stx L0087
            beq LAE06
LAE02:      ldx L0087
            beq LAE28
LAE06:      inx
            cpx #$07
            bcc LAE0D
            ldx #$00
LAE0D:      stx L0087
            txa
            asl
            asl
            asl
            adc L0087
            tax
            ldy L0088
            lda #$08
            sta L00A9
LAE1C:      lda LBE2F,X
            sta L0A00,Y
            inx
            iny
            dec L00A9
            bpl LAE1C
LAE28:      rts
LAE29:      lda PORTA
            sta L00A9
            tax
            lda L0084
            stx L0084
            eor L00A9
            and L00A9
            ror
            rts
LAE39:      lda #$00
            ldx #$07
LAE3D:      sta HPOSP0,X
            dex
            bpl LAE3D
            lda #$1E
            ldx #$03
LAE47:      sta PCOLR0,X
            dex
            bpl LAE47
            lda #$00
            sta L00A7
            lda #$08
            sta L00A8
            lda #$00
            ldx #$04
            ldy #$80
LAE5B:      sta (L00A7),Y
            iny
            bne LAE5B
            inc L00A8
            dex
            bne LAE5B
            lda #$00
            sta L0087
            rts
LAE6A:      lda #$AD
            sta L00A7
            lda #$00
            sta L00A8
LAE72:      stx L00A9
            clc
            lda L0E00,Y
            adc L00A9
            sta L00A5
            lda L0F00,Y
            adc #$00
            sta L00A6
            ldy #$00
            lda (L00A7),Y
            tax
            inc L00A7
            bne LAE8E
            inc L00A8
LAE8E:      lda (L00A7),Y
            cmp #$60
            bcs LAE96
            sbc #$1F
LAE96:      sta (L00A5),Y
            iny
            dex
            bne LAE8E
            rts
LAE9D:      sta L00A7
            stx L00A8
            lda #<dlist1
            sta SDLSTL
            lda #>dlist1
            sta SDLSTH
            lda #$C0
            sta COLOR0
            lda #$C6
            sta COLOR1
            lda #$CC
            sta COLOR2
            lda #$00
            sta COLOR4
            lda #$00
            sta L00A5
            lda #$10
            sta L00A6
            ldy #$00
            ldx #$0D
LAECB:      lda (L00A7),Y
            sta (L00A5),Y
            iny
            bne LAECB
            inc L00A8
            inc L00A6
            dex
            bpl LAECB
LAED9:      lda (L00A7),Y
            sta (L00A5),Y
            iny
            cpy #$60
            bne LAED9
            rts
LAEE3:      sta L00A9
            lsr
            lsr
            lsr
            lsr
            lsr
            ora #$08
            jsr select_bank_a ; sta LD500
            lda L00A9
            and #$1F
            ora #$40 ; bank now starts at $4000 not $8000
            sta L00A8
            lda #$00
            sta L00A7
            lda #$08
            sta L00AC
            lda #$20
            sta L00AB
            txa
            lsr
            lsr
            sta L00A9
            clc
            lda L0E00,Y
            adc L00A9
            sta L00A5
            lda L0F00,Y
            adc #$00
            sta L00A6
LAF17:      ldx L00AC
            ldy #$00
LAF1B:      lda (L00A7),Y
            sta (L00A5),Y
            iny
            dex
            bne LAF1B
            lda L00A5
            clc
            adc #$28
            sta L00A5
            bcc LAF2E
            inc L00A6
LAF2E:      lda L00A7
            clc
            adc L00AC
            sta L00A7
            bcc LAF39
            inc L00A8
LAF39:      dec L00AB
            bne LAF17
            rts
LAF3E:      lda #$00
            sta L00A7
            lda #$10
            sta L00A8
            ldx #$04
            lda #$00
LAF4A:      tay
LAF4B:      sta (L00A7),Y
            iny
            bne LAF4B
            inc L00A8
            dex
            bpl LAF4A
            rts
LAF56:      sta L00A7
            stx L00A8
            sty L00A6
            ldy #$00
            sty L00A5
            lda #$0E
            jsr select_bank_a ; sta LD500
LAF65:      jsr LAF91
            cmp #$13
            bne LAF81
            jsr LAF91
            cmp #$FF
            beq LAF87
            tax
            jsr LAF91
            cpx #$00
            beq LAF81
LAF7B:      jsr LAF88
            dex
            bne LAF7B
LAF81:      jsr LAF88
            jmp LAF65
LAF87:      rts
LAF88:      sta (L00A5),Y
            inc L00A5
            bne LAF90
            inc L00A6
LAF90:      rts
LAF91:      lda (L00A7),Y
            inc L00A7
            bne LAF99
            inc L00A8
LAF99:      rts
            ldx #$06
LAF9C:      jsr LAFA4
            dex
            dex
            bpl LAF9C
            rts
LAFA4:      lda #$00
            tay
LAFA7:      inc L00A0
            sta L04A0,X
            tya
            sta L04A1,X
            lda #$01
            sta L04A8,X
            sta L04A9,X
            sta L0498,X
            sta L0499,X
            dec L00A0
init:
            rts
            pha
            txa
            pha
            lda #$C0
            ldx #$0C
            sta WSYNC
            sta COLPF2
            stx COLPF1
            pla
            tax
            pla
            rti
            cld
            inc L0083
            lda consol
            tay
            eor L0085
            and L0085
            sty L0085
            lsr
            bcc LAFE7
            inc L00A4
LAFE7:      lsr
            bcs LAFED
            lsr
            bcc LAFF3
LAFED:      lda L00A3
            eor #$FF
            sta L00A3
LAFF3:      lda L00A0
            bne LB046
            ldx #$07
LAFF9:      lda L04A8,X
            beq LB043
            dec L04A8,X
            bne LB043
            dec L0498,X
            bne LB030
            ldy L04A0,X
            bne LB013
            tya
            sta AUDF1,X
            beq LB043
LB013:      txa
            pha
            lda #$04
            sta L00A1
LB019:      lda LB1CD,Y
            sta L0480,X
            txa
            clc
            adc #$08
            tax
            iny
            dec L00A1
            bpl LB019
            pla
            tax
            lda L0480,X
            bne LB03A
LB030:      clc
            lda L0488,X
            adc L0480,X
            sta L0480,X
LB03A:      sta AUDF1,X
            lda L0490,X
            sta L04A8,X
LB043:      dex
            bpl LAFF9
LB046:      jmp XITVBV
LB049:      .byte $18,$20,$31,$41,$4A,$4F,$28,$28
            .byte $63,$29,$20,$31,$39,$38,$37,$20
            .byte $41,$54,$41,$52,$49,$20,$43,$6F
            .byte $72,$70,$2E,$20,$41,$6C,$6C,$20
            .byte $52,$69,$67,$68,$74,$73,$20,$52
            .byte $65,$73,$65,$72,$76,$65,$64,$16
            .byte $4E,$75,$6D,$62,$65,$72,$20,$6F
            .byte $66,$20,$53,$68,$6F,$74,$73,$20
            .byte $46,$69,$72,$65,$64,$3A,$16,$4E
            .byte $75,$6D,$62,$65,$72,$20,$6F,$66
            .byte $20,$54,$61,$72,$67,$65,$74,$73
            .byte $20,$48,$69,$74,$3A,$11,$50,$65
            .byte $72,$63,$65,$6E,$74,$20,$41,$63
            .byte $63,$75,$72,$61,$63,$79,$3A,$0B
            .byte $57,$61,$76,$65,$20,$53,$63,$6F
            .byte $72,$65,$3A,$0F,$41,$63,$63,$75
            .byte $72,$61,$63,$79,$20,$42,$6F,$6E
            .byte $75,$73,$3A,$0C,$54,$6F,$74,$61
            .byte $6C,$20,$53,$63,$6F,$72,$65,$3A
            .byte $1A,$50,$6C,$61,$79,$65,$72,$20
            .byte $31,$20,$20,$2D,$20,$53,$54,$41
            .byte $54,$55,$53,$20,$2D,$20,$20,$57
            .byte $61,$76,$65,$1A,$50,$6C,$61,$79
            .byte $65,$72,$20,$32,$20,$20,$2D,$20
            .byte $53,$54,$41,$54,$55,$53,$20,$2D
            .byte $20,$20,$57,$61,$76,$65,$19,$50
            .byte $72,$65,$73,$73,$20,$54,$72,$69
            .byte $67,$67,$65,$72,$20,$74,$6F,$20
            .byte $43,$6F,$6E,$74,$69,$6E,$75,$65
            .byte $15,$59,$6F,$75,$72,$20,$46,$69
            .byte $6E,$61,$6C,$20,$52,$61,$74,$69
            .byte $6E,$67,$20,$69,$73,$3A,$06,$43
            .byte $2E,$45,$2E,$4F,$2E,$09,$50,$72
            .byte $65,$73,$69,$64,$65,$6E,$74,$0E
            .byte $56,$69,$63,$65,$20,$50,$72,$65
            .byte $73,$69,$64,$65,$6E,$74,$08,$44
            .byte $69,$72,$65,$63,$74,$6F,$72,$07
            .byte $4D,$61,$6E,$61,$67,$65,$72,$06
            .byte $48,$61,$63,$6B,$65,$72,$08,$48
            .byte $6F,$62,$62,$79,$69,$73,$74,$0A
            .byte $50,$72,$6F,$67,$72,$61,$6D,$6D
            .byte $65,$72,$08,$45,$6E,$67,$69,$6E
            .byte $65,$65,$72,$0A,$54,$65,$63,$68
            .byte $6E,$69,$63,$69,$61,$6E,$0F,$54
            .byte $72,$6F,$75,$62,$6C,$65,$20,$53
            .byte $68,$6F,$6F,$74,$65,$72
LB1B7:      .byte $47
LB1B8:      .byte $B1,$4E,$B1,$58,$B1,$67,$B1,$70
            .byte $B1,$78,$B1,$7F,$B1,$88,$B1,$93
            .byte $B1,$9C,$B1,$A7,$B1
LB1CD:      .byte $00,$A0,$04,$04,$10,$00,$86,$FE
            .byte $10,$04,$00,$C0,$04,$04,$10,$00
            .byte $86,$FE,$10,$04,$00,$01,$00,$1E
            .byte $01,$00,$05,$00,$1E,$01,$00,$43
            .byte $03,$01,$05,$24,$40,$02,$01,$07
            .byte $29,$40,$02,$01,$05,$2E,$40,$01
            .byte $01,$05,$33,$40,$01,$01,$05,$00
            .byte $0D,$FF,$02,$0C,$00,$1A,$FF,$01
            .byte $11,$42,$09,$00,$07,$01,$00,$0F
            .byte $FF,$01,$08,$4C,$0C,$FF,$01,$08
            .byte $51,$09,$FF,$01,$08,$00,$AF,$F9
            .byte $01,$03,$5B,$AD,$FA,$01,$03,$60
            .byte $AB,$FB,$01,$03,$65,$A9,$FC,$01
            .byte $03,$6A,$A7,$FD,$01,$03,$6F,$A5
            .byte $FE,$01,$03,$00,$AF,$FD,$01,$06
            .byte $79,$AC,$FD,$01,$05,$7E,$A9,$FD
            .byte $01,$04,$83,$A6,$FD,$01,$03,$88
            .byte $A3,$FD,$01,$02,$00,$AC,$FF,$02
            .byte $0C,$00
LB25F:      .byte $38,$3D,$47,$47,$38,$3D,$38,$47
LB267:      .byte $74,$74,$74,$56,$56,$8D,$8D,$8D
            .byte $80,$04,$04,$00,$04,$01,$C0,$04
            .byte $00
LB278:      .byte $00,$00,$10,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$10,$0D,$10
            .byte $15,$15,$12,$10,$0D,$0B,$0E,$0D
            .byte $0B,$0E,$12,$13,$14,$12,$10,$0D
            .byte $0D,$0D,$10,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$10,$10,$10
            .byte $06,$06,$08,$0C,$11,$14,$19,$19
            .byte $1B,$18,$12,$0D,$0A,$08,$05,$06
            .byte $06,$06,$10,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$08,$08,$10
            .byte $16,$16,$10,$10,$15,$16,$08,$09
            .byte $09,$10,$08,$08,$08,$08,$10,$18
            .byte $18,$18,$10,$08,$08,$08,$08,$08
            .byte $08,$10,$18,$18,$18,$10,$18,$08
            .byte $18,$08,$08,$08,$0D,$10,$13,$14
            .byte $12,$10,$0D,$0B,$0E,$0D,$0B,$0D
            .byte $10,$14,$14,$12,$10,$0D,$12,$13
            .byte $0D,$0D,$0D,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$10,$08,$10
            .byte $18,$18,$18,$10,$08,$08,$08,$08
            .byte $08,$08,$10,$18,$18,$18,$10,$08
            .byte $08,$08,$0C,$11,$16,$14,$12,$0E
            .byte $0A,$0C,$0E,$0A,$0B,$0D,$11,$15
            .byte $15,$13,$0F,$0A,$0D,$12,$0C,$0C
            .byte $0C,$0C,$14,$10,$10,$10,$10,$10
LB348:      .byte $00,$00,$10,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$10,$09,$0A
            .byte $0C,$11,$16,$15,$14,$10,$0D,$0C
            .byte $10,$15,$14,$12,$0E,$0A,$0B,$09
            .byte $09,$09,$10,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$10,$10,$10
            .byte $10,$0D,$08,$05,$05,$05,$07,$0D
            .byte $10,$18,$18,$18,$18,$18,$14,$10
            .byte $10,$10,$10,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$14,$0A,$08
            .byte $0C,$14,$16,$14,$12,$0B,$10,$0D
            .byte $14,$18,$14,$14,$14,$08,$08,$08
            .byte $10,$18,$18,$18,$10,$08,$08,$10
            .byte $18,$18,$18,$10,$08,$08,$18,$18
            .byte $08,$08,$08,$08,$0B,$0B,$0B,$11
            .byte $14,$14,$14,$10,$0B,$0A,$10,$15
            .byte $14,$15,$10,$0B,$0B,$15,$15,$0B
            .byte $0B,$0B,$0B,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$10,$10,$10
            .byte $10,$10,$10,$10,$10,$10,$08,$08
            .byte $08,$10,$18,$18,$18,$10,$08,$08
            .byte $10,$18,$18,$18,$10,$08,$08,$08
            .byte $08,$08,$0A,$0A,$0D,$11,$14,$14
            .byte $13,$0F,$0D,$0C,$10,$14,$14,$13
            .byte $10,$0A,$0A,$13,$13,$0D,$0A,$0A
            .byte $0A,$0F,$0F,$10,$10,$10,$10,$10
LB418:      .byte $D8
LB419:      .byte $B4,$FA,$B4,$D8,$B4,$FA,$B4,$20
            .byte $B5,$44,$B5,$20,$B5,$44,$B5,$73
            .byte $B5,$A5,$B5,$D7,$B5,$14,$B6,$51
            .byte $B6,$6D,$B6,$8D,$B6,$A9,$B6,$73
            .byte $B5,$A5,$B5,$D7,$B5,$14,$B6,$51
            .byte $B6,$6D,$B6,$8D,$B6,$A9,$B6,$8D
            .byte $B6,$C9,$B6,$E4,$B6,$1D,$B7,$56
            .byte $B7,$76,$B7,$96,$B7,$BA,$B7,$C9
            .byte $B6,$E4,$B6,$1D,$B7,$56,$B7,$76
            .byte $B7,$96,$B7,$BA,$B7,$E9,$B7,$2A
            .byte $B8,$67,$B8,$A1,$B8,$D4,$B8,$51
            .byte $B9,$88,$B9,$C6,$B9,$03,$BA,$E9
            .byte $B7,$2A,$B8,$67,$B8,$A1,$B8,$D4
            .byte $B8,$15,$B9,$51,$B9,$88,$B9,$C6
            .byte $B9,$03,$BA,$4B,$BA,$76,$BA,$09
            .byte $BB,$9C,$BB,$CB,$BB,$EB,$BC,$4B
            .byte $BA,$76,$BA,$09,$BB,$9C,$BB,$CB
            .byte $BB,$FA,$BB,$2E,$BC,$5E,$BC,$90
            .byte $BC,$C2,$BC,$EB,$BC,$14,$BD,$47
            .byte $BD,$7A,$BD,$AD,$BD,$E3,$BD,$14
            .byte $BD,$47,$BD,$8D,$B6,$A9,$B6,$96
            .byte $B7,$BA,$B7,$4B,$BA,$C9,$B6,$2A
            .byte $B8,$15,$B9,$CB,$BB,$90,$BC,$E3
            .byte $BD,$E3,$BD,$73,$B5,$A5,$B5,$04
            .byte $0A,$06,$06,$06,$8A,$01,$8B,$03
            .byte $04,$02,$85,$02,$80,$07,$06,$85
            .byte $02,$C0,$03,$04,$02,$85,$02,$04
            .byte $0D,$8A,$00,$08,$08,$08,$04,$01
            .byte $00,$04,$0A,$06,$06,$06,$8A,$01
            .byte $8B,$03,$04,$02,$85,$02,$04,$09
            .byte $85,$02,$80,$07,$08,$85,$02,$C0
            .byte $03,$04,$02,$85,$02,$04,$0D,$8A
            .byte $00,$08,$08,$08,$04,$01,$00,$04
            .byte $1F,$06,$06,$8A,$01,$8B,$10,$80
            .byte $02,$04,$0E,$85,$02,$81,$07,$06
            .byte $85,$02,$C1,$03,$C0,$0B,$04,$0E
            .byte $85,$02,$04,$21,$8A,$00,$08,$08
            .byte $04,$01,$00,$04,$1F,$06,$06,$8A
            .byte $01,$8B,$10,$04,$16,$85,$02,$80
            .byte $02,$04,$17,$85,$02,$81,$07,$06
            .byte $85,$02,$C1,$03,$C0,$0B,$04,$17
            .byte $85,$02,$08,$85,$02,$04,$0E,$85
            .byte $02,$04,$21,$8A,$00,$08,$08,$04
            .byte $01,$00,$04,$22,$06,$06,$06,$8A
            .byte $01,$8B,$03,$85,$03,$80,$02,$04
            .byte $26,$85,$02,$06,$85,$02,$C0,$07
            .byte $04,$25,$85,$02,$80,$02,$04,$28
            .byte $85,$01,$04,$25,$85,$01,$C0,$08
            .byte $85,$14,$04,$25,$8A,$00,$08,$08
            .byte $08,$04,$01,$00,$04,$29,$06,$06
            .byte $06,$8A,$01,$8B,$03,$85,$03,$04
            .byte $2C,$85,$02,$80,$02,$04,$2D,$85
            .byte $02,$06,$85,$02,$C0,$07,$80,$02
            .byte $04,$2F,$85,$02,$04,$2C,$85,$02
            .byte $C0,$08,$85,$14,$04,$2C,$8A,$00
            .byte $08,$08,$08,$04,$01,$00,$04,$22
            .byte $06,$06,$06,$8A,$01,$8B,$03,$85
            .byte $03,$80,$02,$04,$26,$85,$02,$06
            .byte $85,$02,$C0,$07,$04,$25,$85,$02
            .byte $80,$02,$04,$28,$85,$01,$04,$25
            .byte $85,$01,$C0,$08,$85,$0A,$80,$02
            .byte $04,$26,$85,$02,$06,$85,$02,$C0
            .byte $07,$04,$25,$8A,$00,$08,$08,$08
            .byte $04,$01,$00,$04,$29,$06,$06,$06
            .byte $8A,$01,$8B,$03,$85,$03,$80,$02
            .byte $04,$2D,$85,$02,$06,$85,$02,$C0
            .byte $07,$04,$2C,$85,$02,$80,$02,$04
            .byte $2F,$85,$01,$04,$2C,$85,$01,$C0
            .byte $08,$85,$0A,$80,$02,$04,$2D,$85
            .byte $02,$06,$85,$02,$C0,$07,$04,$2C
            .byte $8A,$00,$08,$08,$08,$04,$01,$00
            .byte $04,$3F,$06,$06,$8A,$01,$8B,$05
            .byte $04,$30,$85,$02,$80,$0E,$06,$85
            .byte $02,$C0,$03,$04,$41,$8A,$00,$08
            .byte $08,$04,$01,$00,$04,$3F,$06,$06
            .byte $8A,$01,$8B,$05,$81,$02,$04,$30
            .byte $85,$02,$80,$0E,$06,$85,$02,$C0
            .byte $03,$C1,$0B,$04,$41,$8A,$00,$08
            .byte $08,$04,$01,$00,$04,$3F,$06,$06
            .byte $8A,$01,$8B,$20,$04,$30,$85,$01
            .byte $80,$0E,$06,$85,$01,$C0,$03,$04
            .byte $41,$8A,$00,$08,$08,$04,$01,$00
            .byte $04,$3F,$06,$06,$8A,$01,$8B,$20
            .byte $81,$02,$04,$30,$85,$01,$80,$0E
            .byte $06,$85,$01,$C0,$03,$C1,$0B,$04
            .byte $41,$8A,$00,$08,$08,$04,$01,$00
            .byte $04,$4A,$06,$06,$8A,$01,$8B,$01
            .byte $04,$42,$85,$04,$06,$85,$02,$08
            .byte $85,$04,$04,$4C,$8A,$00,$08,$08
            .byte $04,$01,$00,$04,$4A,$06,$06,$8A
            .byte $01,$8B,$01,$04,$42,$85,$04,$06
            .byte $85,$02,$08,$85,$04,$04,$44,$85
            .byte $04,$06,$85,$02,$08,$85,$04,$04
            .byte $46,$85,$04,$06,$85,$02,$08,$85
            .byte $04,$04,$48,$85,$04,$06,$85,$02
            .byte $08,$85,$04,$04,$4C,$8A,$00,$08
            .byte $08,$04,$01,$00,$04,$4A,$06,$06
            .byte $8A,$01,$8B,$01,$04,$42,$85,$04
            .byte $06,$85,$02,$08,$85,$04,$04,$48
            .byte $85,$04,$06,$85,$02,$08,$85,$04
            .byte $04,$46,$85,$04,$06,$85,$02,$08
            .byte $85,$04,$04,$44,$85,$04,$06,$85
            .byte $02,$08,$85,$04,$04,$4C,$8A,$00
            .byte $08,$08,$04,$01,$00,$04,$5A,$06
            .byte $06,$8A,$01,$8B,$05,$04,$4D,$85
            .byte $02,$80,$05,$06,$85,$02,$C0,$03
            .byte $04,$4D,$85,$02,$04,$5C,$8A,$00
            .byte $08,$08,$04,$01,$00,$04,$5A,$06
            .byte $06,$8A,$01,$8B,$05,$04,$53,$85
            .byte $02,$80,$05,$06,$85,$02,$C0,$03
            .byte $04,$4D,$85,$02,$04,$5C,$8A,$00
            .byte $08,$08,$04,$01,$00,$04,$5A,$06
            .byte $06,$8A,$01,$8B,$20,$81,$02,$04
            .byte $4D,$85,$01,$80,$05,$06,$85,$01
            .byte $C0,$03,$C1,$0B,$04,$4D,$85,$01
            .byte $04,$5C,$8A,$00,$08,$08,$04,$01
            .byte $00,$04,$5A,$06,$06,$8A,$01,$8B
            .byte $20,$04,$53,$85,$01,$04,$54,$85
            .byte $01,$80,$05,$06,$85,$01,$C0,$03
            .byte $04,$54,$85,$01,$80,$04,$06,$85
            .byte $01,$C0,$03,$04,$4D,$85,$01,$04
            .byte $5C,$8A,$00,$08,$08,$04,$01,$00
            .byte $04,$71,$06,$06,$8A,$01,$8B,$20
            .byte $04,$5D,$85,$02,$80,$04,$06,$85
            .byte $02,$C0,$03,$04,$6E,$85,$02,$04
            .byte $6A,$85,$02,$80,$03,$06,$85,$02
            .byte $C0,$03,$04,$66,$85,$02,$80,$07
            .byte $06,$85,$02,$C0,$03,$04,$66,$85
            .byte $02,$08,$85,$02,$04,$5D,$85,$02
            .byte $04,$73,$8A,$00,$08,$08,$04,$01
            .byte $00,$04,$71,$06,$06,$8A,$01,$8B
            .byte $20,$04,$5D,$85,$02,$80,$02,$06
            .byte $85,$02,$C0,$03,$04,$70,$85,$02
            .byte $04,$6C,$85,$02,$06,$85,$02,$04
            .byte $66,$85,$02,$80,$07,$06,$85,$02
            .byte $C0,$03,$04,$66,$85,$02,$08,$85
            .byte $02,$04,$5D,$85,$02,$04,$73,$8A
            .byte $00,$08,$08,$04,$01,$00,$04,$71
            .byte $06,$06,$8A,$01,$8B,$20,$04,$5D
            .byte $85,$02,$04,$65,$85,$02,$80,$03
            .byte $06,$85,$02,$C0,$03,$04,$6F,$85
            .byte $02,$04,$63,$85,$02,$06,$85,$02
            .byte $04,$5D,$85,$02,$80,$07,$06,$85
            .byte $02,$C0,$03,$04,$5D,$85,$02,$04
            .byte $73,$8A,$00,$08,$08,$04,$01,$00
            .byte $04,$71,$06,$06,$8A,$01,$8B,$20
            .byte $04,$5D,$85,$02,$04,$65,$85,$02
            .byte $80,$07,$06,$85,$02,$C0,$03,$04
            .byte $70,$85,$02,$04,$5F,$85,$02,$80
            .byte $05,$06,$85,$02,$C0,$03,$04,$5D
            .byte $85,$02,$04,$73,$8A,$00,$08,$08
            .byte $04,$01,$00,$04,$71,$06,$06,$8A
            .byte $01,$8B,$20,$04,$5D,$85,$02,$80
            .byte $06,$06,$85,$02,$C0,$03,$04,$6F
            .byte $85,$02,$04,$68,$85,$02,$80,$05
            .byte $06,$85,$02,$C0,$03,$04,$66,$85
            .byte $02,$80,$07,$06,$85,$02,$C0,$03
            .byte $04,$66,$85,$02,$08,$85,$02,$04
            .byte $5D,$85,$02,$04,$73,$8A,$00,$08
            .byte $08,$04,$01,$00,$04,$71,$06,$06
            .byte $8A,$01,$8B,$20,$04,$5D,$85,$02
            .byte $06,$85,$02,$06,$85,$02,$04,$70
            .byte $85,$02,$04,$6C,$85,$02,$06,$85
            .byte $02,$04,$66,$85,$02,$80,$07,$06
            .byte $85,$02,$C0,$03,$04,$66,$85,$02
            .byte $08,$85,$02,$04,$5D,$85,$02,$04
            .byte $73,$8A,$00,$08,$08,$04,$01,$00
            .byte $04,$88,$06,$06,$8A,$01,$8B,$10
            .byte $04,$74,$85,$02,$80,$04,$06,$85
            .byte $02,$C0,$03,$04,$85,$85,$02,$04
            .byte $81,$85,$02,$04,$85,$85,$02,$04
            .byte $78,$85,$02,$80,$03,$06,$85,$02
            .byte $C0,$03,$04,$74,$85,$02,$04,$8A
            .byte $8A,$00,$08,$08,$04,$01,$00,$04
            .byte $88,$06,$06,$8A,$01,$8B,$10,$04
            .byte $7C,$85,$02,$80,$05,$06,$85,$02
            .byte $C0,$03,$04,$85,$85,$02,$04,$78
            .byte $85,$02,$04,$85,$85,$02,$04,$81
            .byte $85,$02,$80,$03,$06,$85,$02,$C0
            .byte $03,$04,$7D,$85,$02,$08,$85,$02
            .byte $04,$74,$85,$02,$04,$8A,$8A,$00
            .byte $08,$08,$04,$01,$00,$04,$88,$06
            .byte $06,$8A,$01,$8B,$10,$04,$74,$85
            .byte $02,$80,$06,$06,$85,$02,$C0,$03
            .byte $04,$86,$85,$02,$04,$7F,$85,$02
            .byte $06,$85,$02,$06,$85,$02,$04,$85
            .byte $85,$02,$04,$78,$85,$02,$80,$03
            .byte $06,$85,$02,$C0,$03,$04,$74,$85
            .byte $02,$04,$8A,$8A,$00,$08,$08,$04
            .byte $01,$00,$04,$88,$06,$06,$8A,$01
            .byte $8B,$10,$04,$74,$85,$02,$04,$7C
            .byte $85,$02,$80,$07,$06,$85,$02,$C0
            .byte $03,$04,$87,$85,$02,$04,$76,$85
            .byte $02,$06,$85,$02,$06,$85,$02,$04
            .byte $85,$85,$02,$04,$81,$85,$02,$80
            .byte $03,$06,$85,$02,$C0,$03,$04,$7D
            .byte $85,$02,$08,$85,$02,$04,$74,$85
            .byte $02,$04,$8A,$8A,$00,$08,$08,$04
            .byte $01,$00,$04,$9B,$06,$06,$8A,$01
            .byte $8B,$03,$04,$8B,$85,$01,$80,$03
            .byte $06,$85,$01,$C0,$03,$04,$8D,$85
            .byte $01,$04,$8B,$85,$01,$80,$03,$06
            .byte $85,$01,$C0,$03,$04,$9D,$8A,$00
            .byte $08,$08,$04,$01,$00,$04,$9B,$06
            .byte $06,$8A,$01,$8B,$03,$04,$8B,$85
            .byte $01,$80,$03,$06,$85,$01,$C0,$03
            .byte $04,$8D,$85,$01,$04,$8B,$85,$01
            .byte $80,$03,$06,$85,$01,$C0,$03,$04
            .byte $97,$85,$01,$80,$03,$06,$85,$01
            .byte $C0,$03,$04,$99,$85,$01,$04,$97
            .byte $85,$01,$80,$03,$06,$85,$01,$C0
            .byte $03,$04,$8F,$85,$01,$80,$03,$06
            .byte $85,$01,$C0,$03,$04,$91,$85,$01
            .byte $04,$8F,$85,$01,$80,$03,$06,$85
            .byte $01,$C0,$03,$04,$93,$85,$01,$80
            .byte $03,$06,$85,$01,$C0,$03,$04,$95
            .byte $85,$01,$04,$93,$85,$01,$80,$03
            .byte $06,$85,$01,$C0,$03,$04,$8B,$85
            .byte $01,$80,$03,$06,$85,$01,$C0,$03
            .byte $04,$8D,$85,$01,$04,$8B,$85,$01
            .byte $80,$03,$06,$85,$01,$C0,$03,$04
            .byte $9D,$8A,$00,$08,$08,$04,$01,$00
            .byte $04,$9B,$06,$06,$8A,$01,$8B,$03
            .byte $04,$8B,$85,$01,$80,$03,$06,$85
            .byte $01,$C0,$03,$04,$8D,$85,$01,$04
            .byte $8B,$85,$01,$80,$03,$06,$85,$01
            .byte $C0,$03,$04,$93,$85,$01,$80,$03
            .byte $06,$85,$01,$C0,$03,$04,$95,$85
            .byte $01,$04,$93,$85,$01,$80,$03,$06
            .byte $85,$01,$C0,$03,$04,$8F,$85,$01
            .byte $80,$03,$06,$85,$01,$C0,$03,$04
            .byte $91,$85,$01,$04,$8F,$85,$01,$80
            .byte $03,$06,$85,$01,$C0,$03,$04,$97
            .byte $85,$01,$80,$03,$06,$85,$01,$C0
            .byte $03,$04,$99,$85,$01,$04,$97,$85
            .byte $01,$80,$03,$06,$85,$01,$C0,$03
            .byte $04,$8B,$85,$01,$80,$03,$06,$85
            .byte $01,$C0,$03,$04,$8D,$85,$01,$04
            .byte $8B,$85,$01,$80,$03,$06,$85,$01
            .byte $C0,$03,$04,$9D,$8A,$00,$08,$08
            .byte $04,$01,$00,$04,$AF,$06,$06,$8A
            .byte $01,$8B,$30,$04,$9E,$85,$01,$04
            .byte $A6,$85,$01,$80,$07,$06,$85,$01
            .byte $C0,$03,$04,$A0,$85,$01,$80,$05
            .byte $06,$85,$01,$C0,$03,$04,$9E,$85
            .byte $01,$04,$B1,$8A,$00,$08,$08,$04
            .byte $01,$00,$04,$AF,$06,$06,$8A,$01
            .byte $8B,$30,$04,$9E,$85,$01,$04,$A6
            .byte $85,$01,$80,$05,$06,$85,$01,$C0
            .byte $03,$04,$A2,$85,$01,$80,$03,$06
            .byte $85,$01,$C0,$03,$04,$9E,$85,$01
            .byte $04,$B1,$8A,$00,$08,$08,$04,$01
            .byte $00,$04,$AF,$06,$06,$8A,$01,$8B
            .byte $30,$04,$9E,$85,$01,$04,$A6,$85
            .byte $01,$80,$05,$06,$85,$01,$C0,$03
            .byte $85,$0A,$80,$03,$06,$85,$01,$C0
            .byte $03,$04,$A7,$85,$01,$08,$85,$01
            .byte $04,$9E,$85,$01,$04,$B1,$8A,$00
            .byte $08,$08,$04,$01,$00,$04,$AF,$06
            .byte $06,$8A,$01,$8B,$30,$04,$9E,$85
            .byte $01,$04,$A6,$85,$01,$80,$07,$06
            .byte $85,$01,$C0,$03,$85,$0A,$06,$85
            .byte $01,$04,$A7,$85,$01,$08,$85,$01
            .byte $04,$9E,$85,$01,$04,$B1,$8A,$00
            .byte $08,$08,$04,$01,$00,$04,$AF,$06
            .byte $06,$8A,$01,$8B,$30,$04,$9E,$85
            .byte $01,$80,$06,$06,$85,$01,$C0,$03
            .byte $04,$A9,$85,$01,$80,$05,$06,$85
            .byte $01,$C0,$03,$04,$A7,$85,$01,$08
            .byte $85,$01,$04,$9E,$85,$01,$04,$B1
            .byte $8A,$00,$08,$08,$04,$01,$00,$04
            .byte $AF,$06,$06,$8A,$01,$8B,$30,$04
            .byte $9E,$85,$01,$80,$04,$06,$85,$01
            .byte $C0,$03,$04,$AB,$85,$01,$80,$03
            .byte $06,$85,$01,$C0,$03,$04,$A7,$85
            .byte $01,$08,$85,$01,$04,$9E,$85,$01
            .byte $04,$B1,$8A,$00,$08,$08,$04,$01
            .byte $00,$04,$AF,$06,$06,$8A,$01,$8B
            .byte $30,$04,$9E,$85,$01,$80,$04,$06
            .byte $85,$01,$C0,$03,$85,$0A,$80,$03
            .byte $06,$85,$01,$C0,$03,$04,$9E,$85
            .byte $01,$04,$B1,$8A,$00,$08,$08,$04
            .byte $01,$00,$04,$AF,$06,$06,$8A,$01
            .byte $8B,$30,$04,$9E,$85,$01,$04,$A6
            .byte $85,$01,$80,$06,$06,$85,$01,$C0
            .byte $03,$85,$0A,$06,$85,$01,$04,$9E
            .byte $85,$01,$04,$B1,$8A,$00,$08,$08
            .byte $04,$01,$00,$04,$C6,$06,$06,$8A
            .byte $01,$8B,$20,$04,$B2,$85,$01,$04
            .byte $BA,$85,$01,$80,$05,$06,$85,$01
            .byte $C0,$03,$04,$C3,$85,$01,$04,$B6
            .byte $85,$01,$80,$03,$06,$85,$01,$C0
            .byte $03,$04,$B2,$85,$01,$04,$C8,$8A
            .byte $00,$08,$08,$04,$01,$00,$04,$C6
            .byte $06,$06,$8A,$01,$8B,$20,$04,$B2
            .byte $85,$01,$04,$BA,$85,$01,$80,$07
            .byte $06,$85,$01,$C0,$03,$04,$C5,$85
            .byte $01,$04,$B4,$85,$01,$80,$05,$06
            .byte $85,$01,$C0,$03,$04,$B2,$85,$01
            .byte $04,$C8,$8A,$00,$08,$08,$04,$01
            .byte $00,$04,$C6,$06,$06,$8A,$01,$8B
            .byte $20,$04,$B2,$85,$01,$04,$BA,$85
            .byte $01,$80,$05,$06,$85,$01,$C0,$03
            .byte $04,$C5,$85,$01,$04,$B6,$85,$01
            .byte $80,$03,$06,$85,$01,$C0,$03,$04
            .byte $B2,$85,$01,$04,$C8,$8A,$00,$08
            .byte $08,$04,$01,$00,$04,$C6,$06,$06
            .byte $8A,$01,$8B,$20,$04,$B2,$85,$01
            .byte $80,$06,$06,$85,$01,$C0,$03,$04
            .byte $C4,$85,$01,$04,$BD,$85,$01,$80
            .byte $05,$06,$85,$01,$C0,$03,$04,$BB
            .byte $85,$01,$08,$85,$01,$04,$B2,$85
            .byte $01,$04,$C8,$8A,$00,$08,$08,$04
            .byte $01,$00,$04,$C9,$85,$01,$8A,$00
            .byte $8B,$10,$80,$05,$06,$85,$01,$08
            .byte $85,$01,$C0,$06,$04,$CB,$06,$06
            .byte $8A,$01,$80,$02,$06,$06,$08,$08
            .byte $85,$01,$C0,$06,$8A,$00,$08,$08
            .byte $08,$04,$01,$00
LBE0D:      .byte $01,$02,$04,$08,$10,$20,$40,$80
LBE15:      .byte $10,$30,$50,$70,$10,$30,$50,$70
LBE1D:      .byte $08,$08,$08,$08,$28,$28,$28,$28
LBE25:      .byte $00
LBE26:      .byte $00,$53,$AC,$D4,$AC,$B9,$AC,$BF
            .byte $AC
LBE2F:      .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$08,$00,$00
            .byte $00,$00,$00,$00,$00,$08,$1C,$08
            .byte $00,$00,$00,$00,$00,$08,$08,$36
            .byte $08,$08,$00,$00,$00,$08,$08,$00
            .byte $63,$00,$08,$08,$00,$08,$08,$00
            .byte $14,$41,$14,$00,$08,$08,$08,$00
            .byte $22,$00,$00,$00,$22,$00,$08,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$00,$00
            .byte $00,$00,$00,$00,$00,$00,$70,$70
            .byte $70,$4D,$00,$10,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$0D,$0D
            .byte $0D,$0D,$0D,$0D,$0D,$0D,$8D,$02
            .byte $41,$75,$BF,$70,$70,$70,$42,$00
            .byte $10,$02,$02,$02,$02,$02,$02,$02
            .byte $02,$02,$02,$02,$02,$02,$02,$02
            .byte $02,$02,$02,$02,$02,$02,$02,$02
            .byte $41,$DA,$BF
            
            .word run
            .byte $00,$FF
            .word main
